package de.jottyfan.bico.modules.next;

import static de.jottyfan.bico.db.public_.Tables.V_CALENDAR;

import java.time.LocalDate;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.jooq.DSLContext;
import org.jooq.Record2;
import org.jooq.SelectSeekStep1;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Repository;

import de.jottyfan.bico.modules.next.model.NextBean;

/**
 *
 * @author jotty
 *
 */
@Repository
public class NextRepository {
	private static final Logger LOGGER = LogManager.getLogger(NextRepository.class);

	@Autowired
	private DSLContext jooq;

	/**
	 * get the next dates from date on
	 *
	 * @param date the date
	 * @return the next dates as beans in a list; an empty list at least
	 */
	public List<NextBean> getNext(LocalDate date) {
		SelectSeekStep1<Record2<String, LocalDate>, LocalDate> sql = jooq
		// @formatter:off
			.selectDistinct(V_CALENDAR.FULLNAME, V_CALENDAR.SLOT_DAY)
			.from(V_CALENDAR)
			.where(V_CALENDAR.SLOT_DAY.ge(date))
			.orderBy(V_CALENDAR.SLOT_DAY.asc());
		// @formatter:on
		LOGGER.trace(sql);
		Iterator<Record2<String, LocalDate>> i = sql.fetch().iterator();
		List<NextBean> list = new ArrayList<>();
		while (i.hasNext()) {
			Record2<String, LocalDate> r = i.next();
			list.add(NextBean.of(r.get(V_CALENDAR.FULLNAME), r.get(V_CALENDAR.SLOT_DAY)));
		}
		return list;
	}

}
