package de.jottyfan.bico.modules.subject;

import static de.jottyfan.bico.db.public_.Tables.T_LESSON_SUBJECT;
import static de.jottyfan.bico.db.public_.Tables.T_SOURCE;
import static de.jottyfan.bico.db.public_.Tables.T_SUBJECT;
import static de.jottyfan.bico.db.public_.Tables.V_LESSON;

import java.util.List;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.jooq.DSLContext;
import org.jooq.DeleteConditionStep;
import org.jooq.InsertValuesStep8;
import org.jooq.SelectConditionStep;
import org.jooq.SelectOrderByStep;
import org.jooq.SelectWhereStep;
import org.jooq.UpdateConditionStep;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Repository;

import de.jottyfan.bico.db.public_.tables.records.TSourceRecord;
import de.jottyfan.bico.db.public_.tables.records.TSubjectRecord;
import de.jottyfan.bico.db.public_.tables.records.VLessonRecord;

/**
 *
 * @author jotty
 *
 */
@Repository
public class SubjectRepository {
	private static final Logger LOGGER = LogManager.getLogger(SubjectRepository.class);

	@Autowired
	private DSLContext jooq;

	public List<VLessonRecord> getSubjects() {
		SelectWhereStep<VLessonRecord> sql = jooq.selectFrom(V_LESSON);
		LOGGER.trace(sql);
		return sql.fetch().stream().toList();
	}

	public List<TSourceRecord> getSources() {
		SelectWhereStep<TSourceRecord> sql = jooq.selectFrom(T_SOURCE);
		LOGGER.trace(sql);
		return sql.fetch().stream().toList();
	}

	public TSubjectRecord getSubject(Integer pkSubject) {
		SelectConditionStep<TSubjectRecord> sql = jooq
		// @formatter:off
			.selectFrom(T_SUBJECT)
			.where(T_SUBJECT.PK_SUBJECT.eq(pkSubject));
		// @formatter:on
		LOGGER.trace(sql);
		return sql.fetchOne();
	}

	public void addSubject(TSubjectRecord bean) {
		InsertValuesStep8<TSubjectRecord, Integer, String, String, String, String, String, String, Integer> sql = jooq
		// @formatter:off
			.insertInto(T_SUBJECT,
									T_SUBJECT.FK_SOURCE,
									T_SUBJECT.THEME,
									T_SUBJECT.SUBTHEME,
									T_SUBJECT.BOOK_PAGES,
									T_SUBJECT.WORKSHEETS,
									T_SUBJECT.BIBLEVERSE,
									T_SUBJECT.NOTES,
									T_SUBJECT.ORDER_NR)
			.values(bean.getFkSource(), bean.getTheme(), bean.getSubtheme(), bean.getBookPages(), bean.getWorksheets(),
					    bean.getBibleverse(), bean.getNotes(), bean.getOrderNr());
		// @formatter:on
		LOGGER.trace(sql);
		sql.execute();
	}

	public void updateSubject(TSubjectRecord bean) {
		UpdateConditionStep<TSubjectRecord> sql = jooq
		// @formatter:off
			.update(T_SUBJECT)
			.set(T_SUBJECT.FK_SOURCE, bean.getFkSource())
			.set(T_SUBJECT.THEME, bean.getTheme())
			.set(T_SUBJECT.SUBTHEME, bean.getSubtheme())
			.set(T_SUBJECT.BOOK_PAGES, bean.getBookPages())
			.set(T_SUBJECT.WORKSHEETS, bean.getWorksheets())
			.set(T_SUBJECT.BIBLEVERSE, bean.getBibleverse())
			.set(T_SUBJECT.NOTES, bean.getNotes())
			.set(T_SUBJECT.ORDER_NR, bean.getOrderNr())
			.where(T_SUBJECT.PK_SUBJECT.eq(bean.getPkSubject()));
		// @formatter:on
		LOGGER.trace(sql);
		sql.execute();
	}

	public TSubjectRecord getBeanIfDeletable(Integer pkSubject) {
		SelectOrderByStep<TSubjectRecord> sql = jooq
		// @formatter:off
			.selectFrom(T_SUBJECT)
			.where(T_SUBJECT.PK_SUBJECT.notIn(jooq
				.select(T_LESSON_SUBJECT.FK_SUBJECT)
				.from(T_LESSON_SUBJECT)))
			.and(T_SUBJECT.PK_SUBJECT.eq(pkSubject));
		// @formatter:off
		LOGGER.trace(sql);
		return sql.fetchOne();
	}

	public void removeSubject(Integer pkSubject) {
		DeleteConditionStep<TSubjectRecord> sql = jooq.deleteFrom(T_SUBJECT).where(T_SUBJECT.PK_SUBJECT.eq(pkSubject));
		LOGGER.trace(sql);
		sql.execute();
	}
}
